from sys import stdin

import ast
import tokenize


__version__ = '0.4.1'

# Method calls which are missing assert_ prefix
NON_EXISTENT_METHOD_CALLS = [
    'assert_calls',
    'called_once',
    'called_with',
    'called_once_with',
    'has_calls',
    'not_called',
]
# Properties which don't exist on mocks
NON_EXISTENT_ASSERTS = [
    'called_once',
    'not_called',
]

MESSAGE_M200 = "M200 {} is a non-existent mock method"
MESSAGE_M201 = "M201 {} is a non-existent mock property"


class MockChecker:
    name = 'flake8-mock'
    version = __version__

    def __init__(self, tree, *args, **kwargs):
        self.tree = tree

    def run(self):
        checks = (self.check_M200, self.check_M201)
        for node in ast.walk(self.tree):
            for check in checks:
                errors = check(node)
                if not errors:
                    continue

                for error in errors:
                    yield error

    def check_M200(self, node):
        """Check for mock functions which are not asserts.

        We want to check for functions which look like mock asserts at first
        glance, but ate not really testing anything e.g.
        some_mock.called_once() should be some_mock.assert_called_once()
        """
        if not isinstance(node, ast.Expr):
            return

        if not isinstance(node.value, ast.Call):
            return

        if not isinstance(node.value.func, ast.Attribute):
            return

        if node.value.func.attr in NON_EXISTENT_METHOD_CALLS:
            yield (
                node.lineno,
                node.col_offset,
                MESSAGE_M200.format(node.value.func.attr),
                type(self)
            )

    def check_M201(self, node):
        """Check for asserts checking attributes which don't exist.

        We want to check if there are no asserts which can look properly
        but don't actually test anything e.g.
        some_mock.assert_once - this is not a boolean attribute
        """
        if not isinstance(node, ast.Assert):
            return

        if not isinstance(node.test, ast.Attribute):
            return

        if node.test.attr in NON_EXISTENT_ASSERTS:
            yield (
                node.lineno,
                node.col_offset,
                MESSAGE_M201.format(node.test.attr),
                type(self)
            )



from setuptools import setup

requires = [
    "flake8 > 3.0.0",
]


def get_version(fname='flake8_mock.py'):
    with open(fname) as f:
        for line in f:
            if line.startswith('__version__'):
                return eval(line.split('=')[-1])


def get_long_description():
    descr = []
    for fname in ('README.rst',):
        with open(fname) as f:
            descr.append(f.read())
    return '\n\n'.join(descr)


setup(
    name='flake8-mock-x2',
    version=get_version(),
    description="Provides checking for non-existent mock methods and properties",
    long_description=get_long_description(),
    keywords=['flake8', 'mock', 'testing'],
    author='Anna Warzecha',
    author_email='anna.warzecha@gmail.com',
    url='https://github.com/aniav/flake8-mock',
    license='GNU',
    py_modules=['flake8_mock'],
    install_requires=requires,
    setup_requires=['pytest-runner'],
    tests_require=[
        'pytest==4.0.1',
        'pytest-flake8dir==1.3.0'
    ],
    test_suite="tests",
    python_requires='>=3.0',
    zip_safe=False,
    entry_points={
        'flake8.extension': [
            'M2 = flake8_mock:MockChecker',
        ],
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Quality Assurance',
    ],
)
